#include <serial.h>
/*
 *  Title:       serial.c
 *  Programmer:  Reid Harrison recoded for ICC11 by Scott Jantz
 *  Date:        July 4,1996
 *  Version:     2
 *  Description:
 *       Serial Port I/O Routines for ICC11 on the 68HC11
 *       init_serial must be called in order to use any of the
 *       functions.  Enables ICC11 programs to use the serial
 *       port without buffalo
 *
 *       include in programs that need to use the serial port for I/O
 *       interface with baud=9600, 8 data bits, 1 stop bit, parity none, no flow control
 *       requires hc11.h and mil.h to work
 */

/*************Functions*******************************/ 

/*
  init_serial

  Initializes the SCI port on the 68HC11 to operate at 9600 baud.  This
  function must be called at the beginning of your program if you wish
  to use any of the functions in this library.

  Example:
	init_serial();
*/
void init_serial()
{
  CLEAR_BIT(SPCR,0x20);
  BAUD = 0xb0;  /* 0xbO is 9600 0x35 is 300 baud */
  SCCR2 = 0x0C;
}


/*
  get_char

  Waits for a character to be received by the serial port, then returns
  its ASCII value.

  Examples:
	x = get_char();
	if (get_char() == 'F') fd(0);
	get_char();
*/
int get_char()
{
  int test = 0;

  while (test == 0) {

	test = SCSR & 0x20;
  }

  return(SCDR & 0x7F);
}


/*
  put_char

  Writes an ASCII character to the serial port.

  Examples:
	put_char(65);
	put_char('A');
*/
void put_char(int outchar)
{
  int test = 0;

  while (test == 0) 
  {
	test = SCSR & 0x80;
  }

  SCDR = outchar;
}

/*
  write

  Writes a string of text to the serial port.  The only control character
  supported is the newline character (\n).

  Examples:
	write("Hello, world!\n");
	write("one\ntwo\nthree\n");
*/
void write(char strng[80])
{
  int index = 0;
  int a=0;

  while(strng[index] != 0) {
	if ((strng[index] == 0x5c) && (strng[index+1] == 0x6e)) 
	{
	  put_char(10);
	  put_char(13); 
	
	  index += 2;
	}
	else {
	  put_char(strng[index]);
	  index++;
	}
  }
}


/*
  put_int

  Writes an integer to the serial port.
*/
void put_int(int number)
{
  int count;
  int first_digit = 0;

  if (number < 0) {
	put_char('-');
	number *= -1;
  }

  count = 0;
  while (number > 9999) {
	number -= 10000;
	count++;
  }

  if (count > 0) {
	put_char(count + 48);
	count = 0;
	first_digit = 1;
  }

  count = 0;
  while (number > 999) {
	number -= 1000;
	count++;
  }

  if (count > 0 || first_digit == 1) {
	put_char(count + 48);
	count = 0;
	first_digit = 1;
  }

  count = 0;
  while (number > 99) {
	number -= 100;
	count++;
  }

  if (count > 0 || first_digit == 1) {
	put_char(count + 48);
	count = 0;
	first_digit = 1;
  }

  count = 0;
  while (number > 9) {
	number -= 10;
	count++;
  }

  if (count > 0 || first_digit == 1) {
	put_char(count + 48);
	count = 0;
  }

  put_char(number + 48);
}

/*
  write_int

  Print an integer to the serial port prefixed by a space and followed
  by a newline.

  Examples:
	write_int(x);
	write_int(analog(7));
*/
void write_int(int number)
{
  put_char(' ');
  put_int(number);
  put_char(13);
  put_char(10);
}
