/************************************************************************
 * 			MEKATRONIX Copyright 1998 		      *
 * Title	irtj.c					                                          		*
 * Programmer	Ivan Zapata	                                    					*
 * Date		October, 1996		                                      				*
 * Version	1			                                               				*
 *					                                                            *
 * Description	Routines that control the timing and data recording for *
 *              the infrared emitters and detectors. OC3 is used for    *
 *		timing control, and it is assumed that the IR emitters            *
 * 		are connected to bit 0 of PORTB. 	                               	*
 *			                                                      					*
 ************************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
#include <analog.h>
/************************************************************************/


/**************************** Constants *********************************/

#define PERIOD 30000

#define RIGHT_EYE 6
#define LEFT_EYE  7
/************************************************************************/


/*************************** Prototypes *********************************/
#pragma interrupt_handler TOC3_isr;
void discharge(void);
void init_ir(void);
void irmod32(void);
void irmod40(void);
void ir_mode(int);
void TOC3_isr(void);
int ir_value(int);
/************************************************************************/


/***************************** Globals **********************************/
int analog_value[8], current_mode, discharge_count;
/************************************************************************/


void discharge(void)
/************************************************************************
 * Function: Allow the ir detectors time to discharge. Analog ports 6  	*
 * and 7 will read as zeroes while detectors are discharging            *
 * Returns:                                                             *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Functions called: None                                               *
 * Notes:                                                               *
 ************************************************************************/
{
	discharge_count = 7;
	analog_value[7] = 0;
	analog_value[6] = 0;
}



void init_ir(void)
/************************************************************************
 * Function: To initialize the infrared emitters and detectors          *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    analog_value[], current_mode, discharge_count          *
 *   Registers:  TOC3, TCTL1, TMSK1                                     *
 * Functions called: None                                               *
 * Notes:                                                               *
 ************************************************************************/
{
	int i;
	INTR_OFF();
        for (i = 0; i < 8; i++)
		analog_value[i] = 0;
	current_mode = 0;          /* Fire IR LED's before reading port */
	discharge_count = 0;

/*        TOC3 = ((PERIOD / 4) - 5000);*/

        TOC3 = PERIOD;
	CLEAR_BIT(TCTL1, 0x30);    /* Interrupt will not affect OC3 pin */
	SET_BIT(TMSK1, 0x20);      /* Turn on OC3 interrupt */
	INTR_ON();
}


void irmod32(void) 
/************************************************************************
 * Function: Modulates PORTB, bit 0 at 32.25kHz	                  			*
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  PORTB                                                  *
 * Functions called: None                                               *
 * Notes: This subroutine hogs the processor for about 2.5ms            *
 ************************************************************************/
{
	asm("ldaa    #1\n"
                        "ldy    #135\n"            /* 2*(# of IR pulses) */
			"staa    4100\n");         /* 4100 = Port B */
	asm("loop4 : ldaa 4100\n"      /* 4 cycles - necessary */
                        "eora #15\n"                /* 2 cycles - necessary eora #1*/
			"staa 4100\n"              /* 4 cycles - necessary */
			"nop\n"
			"nop\n"
			"nop\n"
			"nop\n"
			"nop\n"
			"nop\n"                    /* 14 cycles (7 nops) */
			"next4 : nop");
	asm("dey\n"                    /* 4 cycles - necessary */
			"bne loop4");              /* 3 cycles - necessary */
				                         /* total = 31 cycles => 32.25 kHz*/
        asm("ldaa 4100");              
	asm("anda #254");
        asm("staa 4100");              /* Clear Port B - Turn LEDs off */
} 


void irmod40(void) 
/************************************************************************
 * Function: Modulates PORTB, bit 0 at exactly 40kHz                    *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  PORTB                                                  *
 * Functions called: None                                               *
 * Notes: This subroutine hogs the processor for about 2.5ms            *
 ************************************************************************/
{
	asm("ldaa    #1\n"
            "ldy    #180\n"           /* 2*(# of IR pulses) */
            "staa    4100\n");        /* 4100 = Port B */
	asm("loop2 : ldaa 4100\n"     /* 4 cycles - necessary */
            "eora #1\n"               /* 2 cycles - necessary */
            "staa 4100\n"             /* 4 cycles - necessary */
            "nop\n"
            "nop\n"
            "nop\n"                   /* 8 cycles (4 nops) */
            "next2 : nop");
	asm("dey\n"                   /* 4 cycles - necessary */
            "bne loop2");          /* 3 cycles - necessary */
                                    /* total = 25 cycles => 40 kHz*/
        asm("ldaa 4100");             
	asm("anda #254");
        asm("staa 4100");             /* Clear Port B - Turn LEDs off */
} 


void ir_mode(int mode)
/************************************************************************
 * Function: Will determine whether the IR emitters are fired before    *
 *           the detectors are read, and at which frequency the LED's   *
 *           will pulse.                                                *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: mode                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    current_mode                                           *
 *   Registers:  None                                                   *
 *                                                                      *
 * Functions called: None                                               *
 * Notes: If mode is a 0, TJ will fire the LED's before reading the IR  *
 *        detectors, if mode is a 1, the LED's will not be fired.       *
 ************************************************************************/
{
	current_mode = mode;
}



void TOC3_isr(void) 
/************************************************************************
 * Function: Interrupt handler for IR detection system.                 *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    current_mode, discharge_count                          *
 *   Registers:  TOC3, PORTE                                            *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    analog_value[], discharge_count                        *
 *   Registers:  TOC3, TFLG1                                            *
 * Functions called: irmod40(), irmod32()                               *
 * Notes: None                                                          *
 ************************************************************************/
{
        if ((!(current_mode))&&(!(discharge_count)))
                irmod40();             /* Modulate at 40 kHz */
        else if (current_mode == 3)    /*         or         */
                irmod32();             /*    at 32.25 kHz    */
	
	analog_value[7] = analog(7);  /* Get readings for front 2 IR detectors */
	analog_value[6] = analog(6);  /* and put them in memory                */

	if (discharge_count)
	{
		discharge_count--;
		analog_value[7] = 0;
		analog_value[6] = 0;
	}

	TOC3 += PERIOD;

	CLEAR_FLAG(TFLG1, 0x20);
}


int ir_value(int num)
/************************************************************************
 * Function: Returns the last reading of PORTE[num]                     *
 * Returns: PORTE[num]                                                  *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    analog_value[]                                         *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Functions called: None                                               *
 * Notes: Calling this subroutine is NOT the same as reading PORTE    	*
 *        directly.                                                     *
 ************************************************************************/
{
	int i;
	i = analog_value[num];
	return i;
}
