/************************************************************************
 * 			MEKATRONIX Copyright 1998 		        *
 * Title        clocktj.c                                               *
 * Programmer   Keith L. Doty                                           *
 * Date         March 13, 1998                                          *
 * Version      1                                                       *
 *                                                                      *
 * Description								*
 *	This file contains all routines and interrupt drivers 	        *
 *      necessary for TJ clock control. The clock keeps track of        *
 *      milliseconds, seconds, minutes, hours, days and timer (timertj).*			*
 ************************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
/************************************************************************/


/**************************** Constants *********************************/


#define CLOCK_TICK 2000   /* 1msec = 2000 clock cycles for 8MHz crystal */ 

/************************************************************************/


/*************************** Prototypes *********************************/

#pragma interrupt_handler TOC5_isr
void init_clocktj(void);
void TOC5_isr(void); /*Interrupt service routine to generate clock ticks*/
void wait(int);   /* Wait for int milliseconds */

/************************************************************************/


/***************************** Globals **********************************/

extern unsigned int msec, seconds, minutes, hours, days, timertj;

/* timertj counts 65,535 milliseconds and rolls over. Can be used
 * therefore to timeout waits of up to 65.535 seconds.
 */

/************************************************************************/


void init_clocktj(void)
/************************************************************************
 * Function: Initializes all necessary variables for the timer.         *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    msec, seconds, minutes, hours, days                    *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    msec, seconds, minutes, hours, days                    *
 *                                                                      *
 * Registers: PACTL, TOC5,TMSK1, TCTL1                                  *
 * Functions called: None                                               *
 * Notes: This routine must be executed to enable the clock.            *
 ************************************************************************/
{

  INTR_OFF();
  CLEAR_BIT(PACTL,0x04);       /* Enable OC5 */
  CLEAR_BIT(TCTL1,0x03);       /* Interrupt will not affect OC5 pin */
  msec=seconds=minutes=hours=days=timertj= 0;
  TOC5 = 0;
  SET_BIT(TMSK1,0x08);         /* Enable OC5 interrupt */
  INTR_ON();
}

/***********************End of init_clocktj ****************************/


void wait(int nmsec)
/**********************************************************************
 * Function Description: Consumes nmsec milliseconds of time.         *
 * Returns: None                                                      *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: nmsec, the number of milliseconds to pause.          *
 *   Globals:    timertj, millisecond counter                         *
 * Outputs                                                            *
 *   None                                                             *
 * Functions called: None                                             *
 * Notes: None                                                        *
 **********************************************************************/
{
unsigned int mark;
  mark = timertj+nmsec;
  if(mark >= nmsec) /* Check for unsigned integer overflow */
    while (timertj < mark); /* Wait for timer to count nmsec. */
  else /* Compensate for overflow */
    { while (!(0 == timertj)); while (timertj < mark); }

}
/***********************End Function wait ****************************/



void TOC5_isr(void)
/************************************************************************
 * Function:                                                            *
 *   Interrupt service routine to generate clock time variables. This   *
 *   isr executes every millisecond and increments the global variables *
 *   msec, seconds, minutes, hours, days appropriately.                 *
 *                                					*
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    msec, seconds, minutes, hours, days                    *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    msec, seconds, minutes, hours, days                    *
 *                                                                      *
 * Registers:  TOC5, TFLG1             	         			*
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
   TOC5 += CLOCK_TICK;     /* Set the timer to interrupt again in 1msec. */
   ++msec ;               /* Increment the msec timer */
   if (msec == 1000) {msec = 0; ++seconds;};
   if (seconds == 60){seconds = 0; ++minutes;};
   if (minutes == 60) {minutes = 0; ++hours;};
   if (hours == 24) {hours = 0; ++days;};

   ++timertj;           /* This is TJ's millisecond timer counter. */

  CLEAR_FLAG(TFLG1,0x08);          /* Clear OC5I flag */
}
