/************************************************************************
 * 			MEKATRONIX Copyright 1998 		      *
 * Title        motjdr.c                                                *
 * Programmer   Keith L. Doty                                           *
 * Date         February 9, 1998                                        *
 * Version      1                                                       *
 *                                                                      *
 * Description								*
 *	This file contains all routines and interrupt drivers 	        *
 *      necessary for TJ to control the wheel motors. OC2 is used       * 
 *      to interrupt the processor and generate the PWM control. The    *
 *      PWM output control pins used are bits PB7(right wheel motor) 	*
 *	and PB6(left wheel motor)of PORTB. Each wheel motor is given a  *
 *      pulse every 15ms, with a pulse width varying from 1 to 2 ms,     *
 *      depending on the desired  motor speed and direction. OC2 is     *
 *      shared by both motors.                                          *
 ************************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
/************************************************************************/


/**************************** Constants *********************************/

#define HALFPERIOD 15000          /* 15msec refresh rate for each motor */

/* 1msec = 2000 clock cycles for 8MHz crystal */
#define MIN_PWM      2200     
#define MAX_PWM      3800
/* The next constant defines the slope of the linear speed equation */
#define SPEED_SLOPE  (MAX_PWM-MIN_PWM)/25  /* SPEED_SLOPE = 80 */
/* A pulse width of 3000 ~stops the motor. */
#define SPEED_INT    3000	
/************************************************************************/


/*************************** Prototypes *********************************/
#pragma interrupt_handler TOC2_isr
void init_motortj(void);
void motortj(int, int);   /* User motor command */
void TOC2_isr(void);	  /* Interrupt service routine to generate PWM*/
/************************************************************************/


/***************************** Globals **********************************/
unsigned int width[2];       /* width[] is the on-time for each motor*/
unsigned int current_width;
char signal_state;	     /* Controls on/off of motor PWM */
char motortjmask[2]; 	     /* Determines motor to be changed */
/************************************************************************/


void init_motortj(void)
/************************************************************************
 * Function: Initializes all necessary variables for the motors.        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:   width[], signal_state, motortjmask[], current_width     *
 *   Registers: TOC2, PORTB, TMSK1, TCTL1                               *
 * Functions called: None                                               *
 * Notes: This routine must be executed before motortj is called.	*
 ************************************************************************/
{

  INTR_OFF();
  CLEAR_BIT(TCTL1,0xC0);          /* Interrupt will not affect OC2 pin */

  width[0] = width[1] = 0;  /* Motors start turned off */
  TOC2 = 0;                       
  signal_state = 0;
  current_width = 0;
  PORTB &= 0x3F; 
  motortjmask[0] = 0x40;
  motortjmask[1] = 0x80;

  SET_BIT(TMSK1,0x40);            /* Enable OC2 interrupt */
  INTR_ON();
}


void motortj(int index, int newwidth)
/************************************************************************
 * Function: Set the motor specified by index to the speed defined	*
 *	     by newwidth, the pulse width of the PWM control signal.	*
 *	     If newwidth not between 0 and 100, or if the index is not 0*
 *	     or 1, the command is ignored. 				*
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: index: index = 0 = left motor; index = 1 = right motor * 
 *		 newwidth: 0<=newwidth<=100, percent duty cycle of PWM  *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals: width[index]                                              *
 *   Registers:  None                   	                        *
 * Functions called: None                                               *
 * Notes:                                                               *
   The left and right motors must turn with opposite rotations to move	*
   the robot in the same direction, hence, the reason for the following	*
   "if statement". "newwidth" is sensitive to sign. A negative value	*
   will reverse the direction of the motor selected by "index".		*
 ************************************************************************/
{
  if(!((index==0)||(index==1))) return;	/* Is "index" in bounds? */
  
  if ((newwidth != 0)&&(newwidth>-101)&&(newwidth < 101)) 
  { /* Execute next statement if speed given in % and not zero. */
    if (index)
      newwidth = (((SPEED_SLOPE*newwidth) >> 3) + SPEED_INT);
    else
      newwidth = (((-SPEED_SLOPE*newwidth) >> 3) + SPEED_INT);
  }
  width[index] = newwidth;
  
}



void TOC2_isr(void)
/************************************************************************
 * Function: Interrupt service routine to generate the PWM motor signals*
 *	     on PB6 and PB7. The PB6 and PB7 PWM signals are phase      *
 *	     shifted "HALFPERIOD" of the motor control cycle. This	*
 *	     function executes four different actions depending on the  *
 *	     "signal_state".						*
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    width[], signal_state, motortjmask[],			*
 *		 current_width : saves previous PWM pulse width         *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    current_width                                          *
 *   Registers:  TOC2, PORTB, TFLG1      				*
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  char odd, index;
  unsigned int pwidth;

/*
   signal_state = 0  -> Turn on motortj0
   signal_state = 1  -> Turn off motortj0
   signal_state = 2  -> Turn on motortj1
   signal_state = 3  -> Turn off motortj1
*/

  index = (signal_state >> 1);  /* index references current motor */

  odd = (signal_state & 0x01);

  pwidth = width[index];
  
  if ((pwidth == 0)&&(!(odd)))
  {
    TOC2 += HALFPERIOD;
    signal_state++;
  }
  else
  {
    if (odd)
      TOC2 += (HALFPERIOD - current_width);
    else
    {
      TOC2 += pwidth;
      current_width = pwidth;
    }
  
    PORTB ^= motortjmask[index];
  }

  signal_state++;
  if (signal_state == 4)
    signal_state = 0;
  
   
  CLEAR_FLAG(TFLG1,0x40);          /* Clear OC2I flag */
}
