/**********************************************************************
 * 			MEKATRONIX Copyright 1998 		                *
 *  Title:       serialtj.c       *                                   *
 *  Programmer:  Keith L. Doty                                        *
 *  Date:        March 13, 1998                                       *
 *  Version:     2.1                                                  *
 *  Description:                                                      *
 *    Serial Port I/O Routines for ICC11 on the 68HC11                *
 *    init_serial must be called in order to use any of the           *
 *    functions.  Enables ICC11 programs to use the serial            *
 *    port without Buffalo Monitor.                                   *
 *                                                                    *
 *    Include this code in programs that need to use the serial port  *
 *    for I/O.                                                        *
 *                                                                    *
 *    Serial Protocol of this program:                                *
 *      baud=9600, 8 data bits, 1 stop bit, no parity, no flow        *
 *      control.                                                      *
 *                                                                    *
 *    Example:                                                        *
 *      Configure Hyperterm on Win95:                                 *
 *      Direct to COM1, 9600 baud, 8 bits, no parity, 1 stop bit, no  *
 *      flow control.                                                 *
 *                                                                    *
 *   Requires hc11.h and mil.h to work                                *
 *                                                                    *
 *   Version History                                                  *
 *     V1 Written by Reid Harrison                                    *
 *     V2 Minor Modifications by Scott Jantz and Keith L.Doty         *
 *     V2.1 Standardized by Keith L. Doty                             *
 **********************************************************************/


/*************************** Includes ********************************/
#include <hc11.h>
#include <mil.h>
/************************ End of Includes ****************************/

/***************************** Prototypes ****************************/

void init_serial(void); /* Set up the SCI port  */
int get_char(void); /* Read (receive) a character from the SCI port.*/
void put_char(int); /* Write (transmit) a character to the SCI port.*/
void write(char *string); /* Write (transmit) a character string. */
void put_int(int);   /* Write (transmit) an integer to the SCI port. */
void write_int(int);
/* 
 * Write (transmit) an integer to the SCI port prefixed by a space and
 * followed by a newline (carriage_return, line_feed).
 */
int read_int(void); 
/* 
 * Read(receive) an integer from the SCI port
 * followed by a carriage_return and a line_feed.
 */


/************************ End of Prototypes **************************/


void init_serial(void)
/**********************************************************************
 * Function description:                                              *
 *   Initializes the SCI port on the 68HC11 to operate at 9600 baud.  *
 *   This function must be called at the beginning of your program if *
 *   you wish to use any of the functions in this library.            *
 *                                                                    *
 * Example:                                                           *
 *  init_serial();                                                    *
 *                                                                    *
 * Returns: None                                                      *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: None                                                 *
 *   Globals:    None                                                 *
 *   Registers:  SPCR                                                 *
 * Outputs                                                            *
 *   Parameters: None                                                 *
 *   Globals:    None                                                 *
 *   Registers:  BAUD, SPCR, SCCR2                                    *
 *                                                                    *
 * Functions called: CLEAR_BIT                                        *
 * Notes: None                                                        *
 *********************************************************************/
{
  CLEAR_BIT(SPCR,0x20);
  BAUD = 0xb0;  /* 0xbO is 9600 0x35 is 300 baud */
  SCCR2 = 0x0C;
}
/***********************End init_serial ****************************/

int get_char(void)
/**********************************************************************
 * Function description:                                              *
 *   Waits for a character to be received by the serial port, then    *
 *   Returns its ASCII value.                                         *
 *                                                                    *
 *   Examples:                                                        *
 *     x = get_char();                                                *
 *     if (get_char() == 'F') fd(0);                                  *
 *     get_char();                                                    *
 *                                                                    *
 * Returns:  7-bit ASCCI character                                    *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: None                                                 *
 *   Globals:    None                                                 *
 *   Registers:  SCSR, SCDR                                           *
 * Outputs                                                            *
 *  Parameters: None                                                  *
 *  Globals:    None                                                  *
 *  Registers:  None                                                  *
 * Functions called: None                                             *
 * Notes: None                                                        *
 *********************************************************************/
{
  int test = 0;

/* Wait for character */
  while (test == 0) {

    test = SCSR & 0x20;
  }
/* Return 7-bit ASCII by stripping bit 8*/
  return(SCDR & 0x7F);
}

/***********************End get_char *********************************/


void put_char(int outchar)
/**********************************************************************
 * Function description:                                              *
 *   Writes an ASCII character to the serial port.                    *
 *                                                                    *
 * Examples:                                                          *
 *   put_char(65);                                                    *
 *   put_char('A');                                                   *
 *                                                                    *
 * Returns: None                                                      *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: outchar                                              *
 *   Globals:    None                                                 *
 *   Registers:  SCSR                                                 *
 * Outputs                                                            *
 *   Parameters: None                                                 *
 *   Globals:    None                                                 *
 *   Registers:  SCDR                                                 *
 * Functions called: None                                             *
 * Notes: None                                                        *
 **********************************************************************/
{
  int test = 0;
/*Wait for transmitter ready*/
  while (test == 0)
  {
    test = SCSR & 0x80;
  }
/*Transmit character*/
  SCDR = outchar;
}
/************************* End put_char ******************************/


void write(char strng[80])
/**********************************************************************
 * Function description:                                              *
 *    Writes a string of text to the serial port.                     *
 *    The only C write control character code                         *
 *    supported is the newline character (\n).                        *
 *                                                                    *
 *    Since any character can be written, the user can                *
 *    determine and send appropriate protocol characters              *
 *    for a serial device.                                            *
 *                                                                    *
 *  Examples:                                                         *
 *    write("Hello, world!\n");                                       *
 *    write("one\ntwo\nthree\n");                                     *
 *                                                                    *
 * Returns:   NONE                                                    *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: strng                                                *
 *   Globals:    None                                                 *
 *   Registers:  None                                                 *
 * Outputs                                                            *
 *   Parameters: None                                                 *
 *   Globals:    None                                                 *
 *   Registers:  None                                                 *
 * Functions called: put_char                                         *
 * Notes: None                                                        *
**********************************************************************/
{
  int index = 0;
  int a=0;

  while(strng[index] != 0) {
    if ((strng[index] == 0x5c) && (strng[index+1] == 0x6e))
    {
      put_char(0x0d); put_char(0x0c); /*CR and LF*/

      index += 2;
    }
    else {
      put_char(strng[index]);
      index++;
    }
  }
}
/***********************End write ***********************************/


void put_int(int number)
/**********************************************************************
 * Function description: Writes an integer to the SCI port. Method of *
 *   succesive divisions by powers of 10 to obtain most significant   *
 *digits first. Since an integer is less than 65,535, only 5 digits   *
 * must be computed from the input parameter number.                  *
 * Returns: None                                                      *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: number                                               *
 *   Globals:    None                                                 *
 *   Registers:  None                                                 *
 * Outputs                                                            *
 *   Parameters: None                                                 *
 *   Globals:    None                                                 *
 *   Registers:  None                                                 *
 * Functions called: put_char                                         *
 * Notes: None                                                        *
**********************************************************************/
{
  int count;
  int first_digit = 0; /* Determines when zeros are significant */

  if (number < 0) { put_char('-'); number = -number;}

  count = 0;
  while (number > 9999) {number -= 10000; count++;}

  if (count > 0) {put_char(count + 48); first_digit = 1;}

  count = 0;
  while (number > 999) {number -= 1000; count++;}

  if (count > 0 || first_digit == 1)
    {put_char(count + 48); first_digit = 1;}

  count = 0;
  while (number > 99) {number -= 100; count++;}

  if (count > 0 || first_digit == 1)
    {put_char(count + 48); first_digit = 1;}

  count = 0;
  while (number > 9) {number -= 10; count++;}

  if (count > 0 || first_digit == 1) put_char(count + 48);

  put_char(number + 48);
}

/***********************End put_int ******************************/

void write_int(int number)
/**********************************************************************
 * Function description:                                              *
    Print an integer to the serial port prefixed by a space and       *
    followed by a newline (carriage_return, line_feed).               *
*                                                                     *
   Examples:                                                          *
     write_int(x);                                                    *
     write_int(analog(7));                                            *
                                                                      *
 * Returns:                                                           *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: number                                               *
 *   Globals:    None                                                 *
 *   Registers:  None                                                 *
 * Outputs                                                            *
 *   Parameters: None                                                 *
 *   Globals:    None                                                 *
 *   Registers:  None                                                 *
 * Functions called: put_char                                         *
 * Notes: None                                                        *
**********************************************************************/
{
  put_char(' ');
  put_int(number);
  put_char(13); put_char(10); /*carriage_return, line_feed*/
}
/***********************End write_int ****************************/



int read_int(void)
/**********************************************************************
 * Function description:                                              *
 *   Receives a sequence of numerical characters from the serial port *
 *   and converts to a signed 16-bit binary number. The number is     *
 *   read modulo 2**15.	The number is echoed as it is typed. A CR and *
 *   
 * Returns:  16-bit signed integer                                    *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: None                                                 *
 *   Globals:    None                                                 *
 *   Registers:  SCSR, SCDR                                           *
 * Outputs                                                            *
 *  Parameters: None                                                  *
 *  Globals:    None                                                  *
 *  Registers:  None                                                  *
 * Functions called: get_char(), put_char                             *
 * Notes: None                                                        *
 *********************************************************************/
{
 int digit, number, sign;

 sign = 0;
 digit = get_char();
 put_char(digit);    /* Echo all entered digits */
 if( digit == '-') {sign = 1; digit = get_char();put_char(digit);}
 if( digit == '+') {sign = 0; digit = get_char();put_char(digit);}

 number = 0;

 while ((digit >= 0x30) && (digit <= 0x39))
   {
 /*Convert digit from ASCII to binary equivalent*/
    digit = digit-0x30;  

 /*Multiply previous number by 10*/
    number = (number<<3) + (number<<1) + digit;  
    digit = get_char();
    put_char(digit);

   } /*end of while*/

 put_char(13); put_char(10);/*carriage-return, line-feed*/

 if(sign==1) return(-number); else return(number);

}

/***********************End read_int **********************************/

